@@ -16,6 +16,8 @@
 #define TOKEN_INVALID_SIGNATURE "eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiIsImtpZCI6IjMifQ.eyJzdHIiOiJncnV0IiwiaW50Ijo0Miwib2JqIjp0cnVlfQ.SgopnfP3vEE7HbuvfyYqZQZZsbu49GBR5w2YCesW7J0i_s5pVYPMIjl6xU4vOs-nV1lEwn7Z_OaQiyEhVftlOUkM5n7w57YViBZkus5C64S6LuQli150oXWNnis4La6qpg_12EocKffvmG940gL2dWg3dnQYenC-fgtX-CNcaIDZUL-NKq3iaQrwvdbuzNADlSBQUfHh80b7uyKgqcT4tboRyAnJXhcjZ-0NWxCIEusnbskmQEqdxEiq28xL8b_F2hDYe5ZuuHw8tmXcXNHUplswEefTCm0phbvi5D490nVBav6ri6zLTkC9IEOR0hA-1f5AYmvsE5NUepLfp6qCsg"
 #define TOKEN_WITH_WHITESPACES "           \v\n\teyJ0eXAiOiJKV1QiLCJhbGciOi\n\n\n   \tJSUzI1NiIsImtpZCI6IjMifQ.eyJz\t\t \v\rdHIiOiJncnV0IiwiaW50I\n\t\vjo0Miwib2JqIjp0cnVlfQ.SgopnfP3vEE7HbuvfyYqZQZZsbu49GBR5w2YCesW7J0i_s5pVYPMIjl6xU4vOs-nV1lEwn7Z_OaQiyEhVftlOUkM5n7w57YViBZkus5C64S6LuQli150oXWNnis4La6qpg_12EocKffvmG940gL2dWg3dnQYenC-fgtX-CNcaIDZUL-NKq3iaQrwvdbuzNADlSBQUfHh80b7uyKgqcT4tboRyAnJXhcjZ-0NWxCIEusnbskmQEqdxEiq28xL8b_F2hDYe5ZuuHw8tmXcXNHUplswEefTCm0phbvi5D490nVBav6ri6zLTkC9\t\n   \vIEOR0hA-1f5AYmvsE5NUepLfpjqCsg     \t\n"
 
+#define TOKEN_UNSECURE "eyJhbGciOiJub25lIn0.eyJzdHIiOiJncnV0IiwiaW50Ijo0Miwib2JqIjp0cnVlfQ."
+
 const char jwk_pubkey_rsa_str[] = "{\"kty\":\"RSA\",\"n\":\"0vx7agoebGcQSuuPiLJXZptN9nndrQmbXEps2aiAFbWhM78LhWx4cbbfAAtVT86zwu1RK7aPFFxuhDR1L6tSoc_BJECPebWKRX"\
                                    "jBZCiFV4n3oknjhMstn64tZ_2W-5JsGY4Hc5n9yBXArwl93lqt7_RN5w6Cf0h4QyQ5v-65YGjQR0_FDW2QvzqY368QQMicAtaSqzs8KJZgnYb9c7d0zgdAZHzu6"\
                                    "qMQvRL5hajrn1n91CbOpbISD08qNLyrdkt-bFTWhAI4vMQFh6WeZu0fM4lFd2NcRwr3XPksINHaQ-G_xBniIqbw0Ls1jF44-csFCur-kEgU8awapJzKnqDKgw\""\
@@ -350,6 +352,45 @@ START_TEST(test_rhonabwy_verify_vulnerabilty_ok)
 }
 END_TEST
 
+START_TEST(test_rhonabwy_jwt_unsecure)
+{
+  jwt_t * jwt;
+  jwk_t * jwk_pubkey_ecdsa;
+  json_t * j_claims = json_pack("{sssiso}", "str", "grut", "int", 42, "obj", json_true());
+  char * token;
+  
+  ck_assert_int_eq(r_jwt_init(&jwt), RHN_OK);
+  ck_assert_int_eq(r_jwk_init(&jwk_pubkey_ecdsa), RHN_OK);
+  
+  ck_assert_int_eq(r_jwt_parse(jwt, TOKEN_UNSECURE, 0), RHN_ERROR_INVALID);
+  ck_assert_int_eq(r_jwt_parse_unsecure(jwt, TOKEN_UNSECURE, 0), RHN_OK);
+  ck_assert_int_eq(r_jwt_get_sign_alg(jwt), R_JWA_ALG_NONE);
+  ck_assert_int_eq(r_jwt_verify_signature(jwt, NULL, 0), RHN_ERROR_INVALID);
+  ck_assert_int_eq(r_jwt_verify_signature(jwt, jwk_pubkey_ecdsa, 0), RHN_ERROR_INVALID);
+  
+  r_jwt_free(jwt);
+  
+  ck_assert_int_eq(r_jwt_init(&jwt), RHN_OK);
+  ck_assert_int_eq(r_jwt_set_full_claims_json_t(jwt, j_claims), RHN_OK);
+  ck_assert_int_eq(r_jwt_set_sign_alg(jwt, R_JWA_ALG_NONE), RHN_OK);
+  ck_assert_ptr_eq(r_jwt_serialize_signed(jwt, NULL, 0), NULL);
+  ck_assert_ptr_eq(r_jwt_serialize_signed(jwt, jwk_pubkey_ecdsa, 0), NULL);
+  ck_assert_ptr_ne(token = r_jwt_serialize_signed_unsecure(jwt, jwk_pubkey_ecdsa, 0), NULL);
+  o_free(token);
+  ck_assert_ptr_ne(token = r_jwt_serialize_signed_unsecure(jwt, NULL, 0), NULL);
+  r_jwt_free(jwt);
+  
+  ck_assert_int_eq(r_jwt_init(&jwt), RHN_OK);
+  ck_assert_int_eq(r_jwt_parse(jwt, token, 0), RHN_ERROR_INVALID);
+  ck_assert_int_eq(r_jwt_parse_unsecure(jwt, token, 0), RHN_OK);
+  ck_assert_int_eq(r_jwt_get_sign_alg(jwt), R_JWA_ALG_NONE);
+  o_free(token);
+  r_jwk_free(jwk_pubkey_ecdsa);
+  r_jwt_free(jwt);
+  json_decref(j_claims);
+}
+END_TEST
+
 static Suite *rhonabwy_suite(void)
 {
   Suite *s;
@@ -369,6 +410,7 @@ static Suite *rhonabwy_suite(void)
   tcase_add_test(tc_core, test_rhonabwy_verify_signature_with_whitespaces);
   tcase_add_test(tc_core, test_rhonabwy_verify_signature_with_add_keys_ok);
   tcase_add_test(tc_core, test_rhonabwy_verify_vulnerabilty_ok);
+  tcase_add_test(tc_core, test_rhonabwy_jwt_unsecure);
   tcase_set_timeout(tc_core, 30);
   suite_add_tcase(s, tc_core);
 
