@@ -24,9 +24,9 @@ START_TEST(test_rhonabwy_info_json_t)
                             "jwe",
                               "alg",
                                 "RSA1_5",
-                                "A128KW",
-                                "A256KW",
                                 "dir",
+                                "A128GCMKW",
+                                "A256GCMKW",
                               "enc",
                                 "A128CBC-HS256",
                                 "A192CBC-HS384",
@@ -38,14 +38,33 @@ START_TEST(test_rhonabwy_info_json_t)
   json_array_append_new(json_object_get(json_object_get(j_info, "jws"), "alg"), json_string("ES384"));
   json_array_append_new(json_object_get(json_object_get(j_info, "jws"), "alg"), json_string("ES512"));
   json_array_append_new(json_object_get(json_object_get(j_info, "jws"), "alg"), json_string("EdDSA"));
+//  json_array_append_new(json_object_get(json_object_get(j_info, "jws"), "alg"), json_string("ES256K"));
   json_array_append_new(json_object_get(json_object_get(j_info, "jws"), "alg"), json_string("PS256"));
   json_array_append_new(json_object_get(json_object_get(j_info, "jws"), "alg"), json_string("PS384"));
   json_array_append_new(json_object_get(json_object_get(j_info, "jws"), "alg"), json_string("PS512"));
 #endif
 #if GNUTLS_VERSION_NUMBER >= 0x03060e
-  json_array_append_new(json_object_get(json_object_get(j_info, "jwe"), "alg"), json_string("A192KW"));
+  json_array_append_new(json_object_get(json_object_get(j_info, "jwe"), "alg"), json_string("A192GCMKW"));
   json_array_append_new(json_object_get(json_object_get(j_info, "jwe"), "enc"), json_string("A192GCM"));
 #endif
+#if NETTLE_VERSION_NUMBER >= 0x030400
+  json_array_append_new(json_object_get(json_object_get(j_info, "jwe"), "alg"), json_string("RSA-OAEP"));
+  json_array_append_new(json_object_get(json_object_get(j_info, "jwe"), "alg"), json_string("RSA-OAEP-256"));
+  json_array_append_new(json_object_get(json_object_get(j_info, "jwe"), "alg"), json_string("A128KW"));
+  json_array_append_new(json_object_get(json_object_get(j_info, "jwe"), "alg"), json_string("A192KW"));
+  json_array_append_new(json_object_get(json_object_get(j_info, "jwe"), "alg"), json_string("A256KW"));
+#endif
+#if GNUTLS_VERSION_NUMBER >= 0x03060d
+  json_array_append_new(json_object_get(json_object_get(j_info, "jwe"), "alg"), json_string("PBES2-HS256+A128KW"));
+  json_array_append_new(json_object_get(json_object_get(j_info, "jwe"), "alg"), json_string("PBES2-HS384+A192KW"));
+  json_array_append_new(json_object_get(json_object_get(j_info, "jwe"), "alg"), json_string("PBES2-HS512+A256KW"));
+#endif
+#if NETTLE_VERSION_NUMBER >= 0x030600
+  json_array_append_new(json_object_get(json_object_get(j_info, "jwe"), "alg"), json_string("ECDH-ES"));
+  json_array_append_new(json_object_get(json_object_get(j_info, "jwe"), "alg"), json_string("ECDH-ES+A128KW"));
+  json_array_append_new(json_object_get(json_object_get(j_info, "jwe"), "alg"), json_string("ECDH-ES+A192KW"));
+  json_array_append_new(json_object_get(json_object_get(j_info, "jwe"), "alg"), json_string("ECDH-ES+A256KW"));
+#endif
 
   ck_assert_ptr_ne(j_info, NULL);
   ck_assert_ptr_ne(j_info_control, NULL);
@@ -72,9 +91,9 @@ START_TEST(test_rhonabwy_info_str)
                             "jwe",
                               "alg",
                                 "RSA1_5",
-                                "A128KW",
-                                "A256KW",
                                 "dir",
+                                "A128GCMKW",
+                                "A256GCMKW",
                               "enc",
                                 "A128CBC-HS256",
                                 "A192CBC-HS384",
@@ -86,13 +105,32 @@ START_TEST(test_rhonabwy_info_str)
   json_array_append_new(json_object_get(json_object_get(j_info, "jws"), "alg"), json_string("ES384"));
   json_array_append_new(json_object_get(json_object_get(j_info, "jws"), "alg"), json_string("ES512"));
   json_array_append_new(json_object_get(json_object_get(j_info, "jws"), "alg"), json_string("EdDSA"));
+//  json_array_append_new(json_object_get(json_object_get(j_info, "jws"), "alg"), json_string("ES256K"));
   json_array_append_new(json_object_get(json_object_get(j_info, "jws"), "alg"), json_string("PS256"));
   json_array_append_new(json_object_get(json_object_get(j_info, "jws"), "alg"), json_string("PS384"));
   json_array_append_new(json_object_get(json_object_get(j_info, "jws"), "alg"), json_string("PS512"));
 #endif
 #if GNUTLS_VERSION_NUMBER >= 0x03060e
-  json_array_append_new(json_object_get(json_object_get(j_info, "jwe"), "alg"), json_string("A192KW"));
+  json_array_append_new(json_object_get(json_object_get(j_info, "jwe"), "alg"), json_string("A192GCMKW"));
   json_array_append_new(json_object_get(json_object_get(j_info, "jwe"), "enc"), json_string("A192GCM"));
+#endif
+#if NETTLE_VERSION_NUMBER >= 0x030400
+  json_array_append_new(json_object_get(json_object_get(j_info, "jwe"), "alg"), json_string("RSA-OAEP"));
+  json_array_append_new(json_object_get(json_object_get(j_info, "jwe"), "alg"), json_string("RSA-OAEP-256"));
+  json_array_append_new(json_object_get(json_object_get(j_info, "jwe"), "alg"), json_string("A128KW"));
+  json_array_append_new(json_object_get(json_object_get(j_info, "jwe"), "alg"), json_string("A192KW"));
+  json_array_append_new(json_object_get(json_object_get(j_info, "jwe"), "alg"), json_string("A256KW"));
+#endif
+#if GNUTLS_VERSION_NUMBER >= 0x03060d
+  json_array_append_new(json_object_get(json_object_get(j_info, "jwe"), "alg"), json_string("PBES2-HS256+A128KW"));
+  json_array_append_new(json_object_get(json_object_get(j_info, "jwe"), "alg"), json_string("PBES2-HS384+A192KW"));
+  json_array_append_new(json_object_get(json_object_get(j_info, "jwe"), "alg"), json_string("PBES2-HS512+A256KW"));
+#endif
+#if NETTLE_VERSION_NUMBER >= 0x030600
+  json_array_append_new(json_object_get(json_object_get(j_info, "jwe"), "alg"), json_string("ECDH-ES"));
+  json_array_append_new(json_object_get(json_object_get(j_info, "jwe"), "alg"), json_string("ECDH-ES+A128KW"));
+  json_array_append_new(json_object_get(json_object_get(j_info, "jwe"), "alg"), json_string("ECDH-ES+A192KW"));
+  json_array_append_new(json_object_get(json_object_get(j_info, "jwe"), "alg"), json_string("ECDH-ES+A256KW"));
 #endif
   json_t * j_info_control_parsed = json_loads(j_info_control_str, JSON_DECODE_ANY, NULL);
 
@@ -122,6 +160,10 @@ START_TEST(test_rhonabwy_alg_conversion)
   ck_assert_str_eq(r_jwa_alg_to_str(R_JWA_ALG_ES384), "ES384");
   ck_assert_int_eq(r_str_to_jwa_alg("ES512"), R_JWA_ALG_ES512);
   ck_assert_str_eq(r_jwa_alg_to_str(R_JWA_ALG_ES512), "ES512");
+  ck_assert_int_eq(r_str_to_jwa_alg("EdDSA"), R_JWA_ALG_EDDSA);
+  ck_assert_str_eq(r_jwa_alg_to_str(R_JWA_ALG_EDDSA), "EdDSA");
+  ck_assert_int_eq(r_str_to_jwa_alg("ES256K"), R_JWA_ALG_ES256K);
+  ck_assert_str_eq(r_jwa_alg_to_str(R_JWA_ALG_ES256K), "ES256K");
   ck_assert_int_eq(r_str_to_jwa_alg("RS256"), R_JWA_ALG_RS256);
   ck_assert_str_eq(r_jwa_alg_to_str(R_JWA_ALG_RS256), "RS256");
   ck_assert_int_eq(r_str_to_jwa_alg("RS384"), R_JWA_ALG_RS384);
