@@ -1,9 +1,9 @@
 /**
- * 
+ *
  * rnbyc: Rhonabwy command-line tool
- * 
+ *
  * Copyright 2020 Nicolas Mora <mail@babelouest.org>
- * 
+ *
  * Command-line tool to manipulate JWK, JWKS, JWE, JWS and JWT
  * - Generates random JWK using the specified algorithm
  * - Generates random JWKS using the specified algorithm
@@ -30,7 +30,7 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program.  If not, see <http://www.gnu.org/licenses/>.
- * 
+ *
  */
 
 #include <string.h>
@@ -48,7 +48,7 @@
 #include <yder.h>
 #include <rhonabwy.h>
 
-#define _RNBYC_VERSION_ "0.9"
+#define _RNBYC_VERSION_ "1.0"
 
 #define R_RSA_DEFAULT_SIZE 4096
 #define R_OCT_DEFAULT_SIZE 128
@@ -58,6 +58,10 @@
 #define R_ACTION_PARSE_TOKEN     2
 #define R_ACTION_SERIALIZE_TOKEN 3
 
+#define RNBYC_FORMAT_JWK 0
+#define RNBYC_FORMAT_PEM 1
+#define RNBYC_FORMAT_DER 2
+
 static void print_help(FILE * output) {
   fprintf(output, "\nrnbyc - Rhonabwy JWK and JWT parser and generator\n");
   fprintf(output, "\n");
@@ -75,7 +79,13 @@ static void print_help(FILE * output) {
   fprintf(output, "-g --generate <type>\n");
   fprintf(output, "\tGenerate a key pair or a symmetric key\n");
   fprintf(output, "\t<type> - values available:\n");
-  fprintf(output, "\tRSA[key size] (default key size: 4096), ECDSA256, ECDSA384, ECDSA512, EDDSA, oct[key size] (default key size: 128 bits)\n");
+#if NETTLE_VERSION_NUMBER >= 0x03060e
+  fprintf(output, "\tRSA[key size] (default key size: 4096), EC256, EC384, EC521, Ed25519, Ed448, X25519, X448, oct[key size] (default key size: 128 bits)\n");
+#elif NETTLE_VERSION_NUMBER >= 0x030600
+  fprintf(output, "\tRSA[key size] (default key size: 4096), EC256, EC384, EC521, Ed25519, X25519, oct[key size] (default key size: 128 bits)\n");
+#else
+  fprintf(output, "\tRSA[key size] (default key size: 4096), EC256, EC384, EC521, oct[key size] (default key size: 128 bits)\n");
+#endif
   fprintf(output, "-i --stdin\n");
   fprintf(output, "\tReads key to parse from stdin\n");
   fprintf(output, "-f --in-file\n");
@@ -95,6 +105,8 @@ static void print_help(FILE * output) {
   fprintf(output, "\tSpecifies the output file for the public keys in the JWKS\n");
   fprintf(output, "-n --indent\n");
   fprintf(output, "\tJWKS output spaces indentation: 0 is compact mode, default is 2 spaces indent\n");
+  fprintf(output, "-F --format\n");
+  fprintf(output, "\tOutput format, values available are JWK (default), PEM or DER\n");
   fprintf(output, "-x --split\n");
   fprintf(output, "\tSplit JWKS output in public and private keys\n");
   fprintf(output, "-t --parse-token\n");
@@ -102,35 +114,37 @@ static void print_help(FILE * output) {
   fprintf(output, "-s --serialize-token\n");
   fprintf(output, "\tAction: serialize given claims in a token\n");
   fprintf(output, "-H --header\n");
-  fprintf(output, "\tDisplay header of a parsed token, default false\n");
+  fprintf(output, "\tDisplay header of a parsed token\n");
   fprintf(output, "-C --claims\n");
   fprintf(output, "\tDisplay claims of a parsed token, default true\n");
   fprintf(output, "-P --public-key\n");
-  fprintf(output, "\tSpecifies the public key to for key management encryption or signature verification\n");
+  fprintf(output, "\tSpecifies the public key for key management encryption or signature verification\n");
   fprintf(output, "\tPublic key must be in JWKS format and can be either a JWKS string or a path to a JWKS file\n");
   fprintf(output, "-K --private-key\n");
-  fprintf(output, "\tSpecifies the private key to for key management decryption or signature generation\n");
+  fprintf(output, "\tSpecifies the private key for key management decryption or signature generation\n");
   fprintf(output, "\tPublic key must be in JWKS format and can be either a JWKS string or a path to a JWKS file\n");
+  fprintf(output, "-W --password\n");
+  fprintf(output, "\tSpecifies the password for key management encryption/decryption using PBES2 alg or signature generation/verification using HS alg\n");
   fprintf(output, "-u --x5u-flags\n");
   fprintf(output, "\tSet x5u flags to retrieve online certificate, values available are:\n");
   fprintf(output, "\t\tcert: ignore server certificate errors (self-signed, expired, etc.)\n");
   fprintf(output, "\t\tfollow: follow jwks_uri redirection if any\n");
   fprintf(output, "\t\tvalues can be contatenated, e.g. --x5u-flags cert,follow\n");
   fprintf(output, "-v --version\n");
-  fprintf(output, "\tPrint uwsc's current version\n");
+  fprintf(output, "\tPrint rnbyc's current version\n");
   fprintf(output, "-h --help\n");
   fprintf(output, "\tPrint this message\n\n");
   fprintf(output, "-d --debug\n");
   fprintf(output, "\tDisplay debug messages\n\n");
 }
 
-static int write_file_content(const char * file_path, const char * content) {
+static int write_file_content(const char * file_path, const char * content, size_t content_len) {
   FILE * f;
   int ret;
 
   f = fopen (file_path, "w+");
   if (f) {
-    if (fwrite(content, 1, o_strlen(content), f) > 0 && fwrite("\n", 1, 1, f) > 0) {
+    if (fwrite(content, 1, content_len, f) > 0 && fwrite("\n", 1, 1, f) > 0) {
       ret = 0;
     } else {
       ret = ENOENT;
@@ -140,7 +154,7 @@ static int write_file_content(const char * file_path, const char * content) {
     fprintf(stderr, "error opening file %s\n", file_path);
     ret = EACCES;
   }
-  
+
   return ret;
 }
 
@@ -167,7 +181,7 @@ static char * get_file_content(const char * file_path) {
   } else {
     fprintf(stderr, "error opening file %s\n", file_path);
   }
-  
+
   return buffer;
 }
 
@@ -192,7 +206,7 @@ static int jwk_generate(jwks_t * jwks_privkey, jwks_t * jwks_pubkey, json_t * j_
   const char * type = json_string_value(json_object_get(j_element, "type"));
   int ret = 0;
   json_int_t bits;
-  
+
   if (r_jwk_init(&jwk_priv) == RHN_OK && r_jwk_init(&jwk_pub) == RHN_OK) {
     if (0 == o_strcmp("RSA", type)) {
       r_jwk_generate_key_pair(jwk_priv, jwk_pub, R_KEY_TYPE_RSA, json_integer_value(json_object_get(j_element, "bits")), json_string_value(json_object_get(j_element, "kid")));
@@ -206,8 +220,8 @@ static int jwk_generate(jwks_t * jwks_privkey, jwks_t * jwks_pubkey, json_t * j_
       } else {
         r_jwks_append_jwk(jwks_privkey, jwk_pub);
       }
-    } else if (0 == o_strcasecmp("ECDSA256", type)) {
-      r_jwk_generate_key_pair(jwk_priv, jwk_pub, R_KEY_TYPE_ECDSA, 256, json_string_value(json_object_get(j_element, "kid")));
+    } else if (0 == o_strcasecmp("EC256", type)) {
+      r_jwk_generate_key_pair(jwk_priv, jwk_pub, R_KEY_TYPE_EC, 256, json_string_value(json_object_get(j_element, "kid")));
       if (json_string_length(json_object_get(j_element, "alg"))) {
         r_jwk_set_property_str(jwk_priv, "alg", json_string_value(json_object_get(j_element, "alg")));
         r_jwk_set_property_str(jwk_pub, "alg", json_string_value(json_object_get(j_element, "alg")));
@@ -221,8 +235,8 @@ static int jwk_generate(jwks_t * jwks_privkey, jwks_t * jwks_pubkey, json_t * j_
       } else {
         r_jwks_append_jwk(jwks_privkey, jwk_pub);
       }
-    } else if (0 == o_strcasecmp("ECDSA384", type)) {
-      r_jwk_generate_key_pair(jwk_priv, jwk_pub, R_KEY_TYPE_ECDSA, 384, json_string_value(json_object_get(j_element, "kid")));
+    } else if (0 == o_strcasecmp("EC384", type)) {
+      r_jwk_generate_key_pair(jwk_priv, jwk_pub, R_KEY_TYPE_EC, 384, json_string_value(json_object_get(j_element, "kid")));
       if (json_string_length(json_object_get(j_element, "alg"))) {
         r_jwk_set_property_str(jwk_priv, "alg", json_string_value(json_object_get(j_element, "alg")));
         r_jwk_set_property_str(jwk_pub, "alg", json_string_value(json_object_get(j_element, "alg")));
@@ -236,8 +250,8 @@ static int jwk_generate(jwks_t * jwks_privkey, jwks_t * jwks_pubkey, json_t * j_
       } else {
         r_jwks_append_jwk(jwks_privkey, jwk_pub);
       }
-    } else if (0 == o_strcasecmp("ECDSA512", type)) {
-      r_jwk_generate_key_pair(jwk_priv, jwk_pub, R_KEY_TYPE_ECDSA, 512, json_string_value(json_object_get(j_element, "kid")));
+    } else if (0 == o_strcasecmp("EC521", type)) {
+      r_jwk_generate_key_pair(jwk_priv, jwk_pub, R_KEY_TYPE_EC, 521, json_string_value(json_object_get(j_element, "kid")));
       if (json_string_length(json_object_get(j_element, "alg"))) {
         r_jwk_set_property_str(jwk_priv, "alg", json_string_value(json_object_get(j_element, "alg")));
         r_jwk_set_property_str(jwk_pub, "alg", json_string_value(json_object_get(j_element, "alg")));
@@ -251,7 +265,8 @@ static int jwk_generate(jwks_t * jwks_privkey, jwks_t * jwks_pubkey, json_t * j_
       } else {
         r_jwks_append_jwk(jwks_privkey, jwk_pub);
       }
-    } else if (0 == o_strcasecmp("EDDSA", type)) {
+#if NETTLE_VERSION_NUMBER >= 0x030600
+    } else if (0 == o_strcasecmp("Ed25519", type)) {
       r_jwk_generate_key_pair(jwk_priv, jwk_pub, R_KEY_TYPE_EDDSA, 256, json_string_value(json_object_get(j_element, "kid")));
       if (json_string_length(json_object_get(j_element, "alg"))) {
         r_jwk_set_property_str(jwk_priv, "alg", json_string_value(json_object_get(j_element, "alg")));
@@ -266,6 +281,54 @@ static int jwk_generate(jwks_t * jwks_privkey, jwks_t * jwks_pubkey, json_t * j_
       } else {
         r_jwks_append_jwk(jwks_privkey, jwk_pub);
       }
+    } else if (0 == o_strcasecmp("X25519", type)) {
+      r_jwk_generate_key_pair(jwk_priv, jwk_pub, R_KEY_TYPE_ECDH, 256, json_string_value(json_object_get(j_element, "kid")));
+      if (json_string_length(json_object_get(j_element, "alg"))) {
+        r_jwk_set_property_str(jwk_priv, "alg", json_string_value(json_object_get(j_element, "alg")));
+        r_jwk_set_property_str(jwk_pub, "alg", json_string_value(json_object_get(j_element, "alg")));
+      } else {
+        r_jwk_set_property_str(jwk_priv, "alg", "X25519");
+        r_jwk_set_property_str(jwk_pub, "alg", "X25519");
+      }
+      r_jwks_append_jwk(jwks_privkey, jwk_priv);
+      if (jwks_pubkey != NULL) {
+        r_jwks_append_jwk(jwks_pubkey, jwk_pub);
+      } else {
+        r_jwks_append_jwk(jwks_privkey, jwk_pub);
+      }
+#endif
+#if NETTLE_VERSION_NUMBER >= 0x03060e
+    } else if (0 == o_strcasecmp("Ed448", type)) {
+      r_jwk_generate_key_pair(jwk_priv, jwk_pub, R_KEY_TYPE_EDDSA, 448, json_string_value(json_object_get(j_element, "kid")));
+      if (json_string_length(json_object_get(j_element, "alg"))) {
+        r_jwk_set_property_str(jwk_priv, "alg", json_string_value(json_object_get(j_element, "alg")));
+        r_jwk_set_property_str(jwk_pub, "alg", json_string_value(json_object_get(j_element, "alg")));
+      } else {
+        r_jwk_set_property_str(jwk_priv, "alg", "EdDSA");
+        r_jwk_set_property_str(jwk_pub, "alg", "EdDSA");
+      }
+      r_jwks_append_jwk(jwks_privkey, jwk_priv);
+      if (jwks_pubkey != NULL) {
+        r_jwks_append_jwk(jwks_pubkey, jwk_pub);
+      } else {
+        r_jwks_append_jwk(jwks_privkey, jwk_pub);
+      }
+    } else if (0 == o_strcasecmp("X448", type)) {
+      r_jwk_generate_key_pair(jwk_priv, jwk_pub, R_KEY_TYPE_ECDH, 448, json_string_value(json_object_get(j_element, "kid")));
+      if (json_string_length(json_object_get(j_element, "alg"))) {
+        r_jwk_set_property_str(jwk_priv, "alg", json_string_value(json_object_get(j_element, "alg")));
+        r_jwk_set_property_str(jwk_pub, "alg", json_string_value(json_object_get(j_element, "alg")));
+      } else {
+        r_jwk_set_property_str(jwk_priv, "alg", "X25519");
+        r_jwk_set_property_str(jwk_pub, "alg", "X25519");
+      }
+      r_jwks_append_jwk(jwks_privkey, jwk_priv);
+      if (jwks_pubkey != NULL) {
+        r_jwks_append_jwk(jwks_pubkey, jwk_pub);
+      } else {
+        r_jwks_append_jwk(jwks_privkey, jwk_pub);
+      }
+#endif
     } else if (0 == o_strcasecmp("oct", type)) {
       bits = json_integer_value(json_object_get(j_element, "bits"));
       bits += (bits%8);
@@ -304,7 +367,7 @@ static int jwks_parse_str(jwks_t * jwks_priv, jwks_t * jwks_pub, const char * in
   int ret, key_type;
   size_t i;
 
-  if (r_jwks_init(&jwks) == RHN_OK && r_jwks_import_from_str(jwks, in) == RHN_OK) {
+  if (r_jwks_init(&jwks) == RHN_OK && r_jwks_import_from_json_str(jwks, in) == RHN_OK) {
     for (i=0; i<r_jwks_size(jwks); i++) {
       jwk = r_jwks_get_at(jwks, i);
       if ((key_type = r_jwk_key_type(jwk, NULL, x5u_flags)) & R_KEY_TYPE_PUBLIC && jwks_pub != NULL) {
@@ -372,7 +435,7 @@ static int jwks_parse_str(jwks_t * jwks_priv, jwks_t * jwks_pub, const char * in
 static int jwk_stdin(jwks_t * jwks_priv, jwks_t * jwks_pub, json_t * j_element, int x5u_flags) {
   char * in = get_stdin_content();
   int ret;
-  
+
   if (in != NULL) {
     ret = jwks_parse_str(jwks_priv, jwks_pub, in, json_string_value(json_object_get(j_element, "kid")), x5u_flags);
   } else {
@@ -385,7 +448,7 @@ static int jwk_stdin(jwks_t * jwks_priv, jwks_t * jwks_pub, json_t * j_element,
 static int jwk_file(jwks_t * jwks_priv, jwks_t * jwks_pub, json_t * j_element, int x5u_flags) {
   char * in = get_file_content(json_string_value(json_object_get(j_element, "path")));
   int ret;
-  
+
   if (in != NULL) {
     ret = jwks_parse_str(jwks_priv, jwks_pub, in, json_string_value(json_object_get(j_element, "kid")), x5u_flags);
   } else {
@@ -395,12 +458,14 @@ static int jwk_file(jwks_t * jwks_priv, jwks_t * jwks_pub, json_t * j_element, i
   return ret;
 }
 
-static void get_jwks_out(json_t * j_arguments, int split_keys, int x5u_flags, int indent, const char * out_file, const char * out_file_public) {
+static void get_jwks_out(json_t * j_arguments, int split_keys, int x5u_flags, int indent, int format, const char * out_file, const char * out_file_public) {
   jwks_t * jwks_privkey = NULL, * jwks_pubkey = NULL;
-  json_t * j_element = NULL, * j_jwks;
-  size_t index = 0;
-  char * str_jwks;
-  
+  jwk_t * cur_jwk;
+  json_t * j_element = NULL, * j_jwks = NULL;
+  size_t index = 0, out_len = 0, str_jwks_len = 0;
+  char * str_jwks = NULL;
+  unsigned char * out = NULL;
+
   if (r_jwks_init(&jwks_privkey) == RHN_OK && r_jwks_init(&jwks_pubkey) == RHN_OK) {
     json_array_foreach(j_arguments, index, j_element) {
       if (0 == o_strcmp("generate", json_string_value(json_object_get(j_element, "source")))) {
@@ -421,30 +486,88 @@ static void get_jwks_out(json_t * j_arguments, int split_keys, int x5u_flags, in
     fprintf(stderr, "Error r_jwks_init\n");
   }
   if (r_jwks_size(jwks_privkey)) {
-    j_jwks = r_jwks_export_to_json_t(jwks_privkey);
-    str_jwks = json_dumps(j_jwks, JSON_INDENT(indent));
-    if (out_file != NULL) {
-      if (write_file_content(out_file, str_jwks)) {
-        fprintf(stderr, "Error writing to file %s\n", out_file);
-      }
+    if (format == RNBYC_FORMAT_JWK) {
+      j_jwks = r_jwks_export_to_json_t(jwks_privkey);
+      str_jwks = json_dumps(j_jwks, JSON_INDENT(indent)|JSON_SORT_KEYS);
+      str_jwks_len = o_strlen(str_jwks);
     } else {
-      if (r_jwks_size(jwks_pubkey)) {
-        printf("Private keys:\n");
+      str_jwks = NULL;
+      for (index=0; index<r_jwks_size(jwks_privkey); index++) {
+        cur_jwk = r_jwks_get_at(jwks_privkey, index);
+        if (r_jwk_export_to_pem_der(cur_jwk, format==RNBYC_FORMAT_PEM?R_FORMAT_PEM:R_FORMAT_DER, NULL, &out_len, x5u_flags) == RHN_ERROR_PARAM) {
+          if ((out = o_malloc(out_len+1)) != NULL) {
+            if (r_jwk_export_to_pem_der(cur_jwk, format==RNBYC_FORMAT_PEM?R_FORMAT_PEM:R_FORMAT_DER, out, &out_len, x5u_flags) == RHN_OK) {
+              str_jwks = o_realloc(str_jwks, str_jwks_len+out_len);
+              memcpy(str_jwks+str_jwks_len, out, out_len);
+              str_jwks_len += out_len;
+            } else {
+              fprintf(stderr, "Error exporting jwks (2)\n");
+            }
+          } else {
+            fprintf(stderr, "Error allocating resources\n");
+          }
+          o_free(out);
+          out_len = 0;
+        } else {
+          fprintf(stderr, "Error exporting jwks (1)\n");
+        }
+        r_jwk_free(cur_jwk);
+      }
+    }
+    if (str_jwks != NULL) {
+      if (out_file != NULL) {
+        if (write_file_content(out_file, str_jwks, str_jwks_len)) {
+          fprintf(stderr, "Error writing to file %s\n", out_file);
+        }
+      } else {
+        if (r_jwks_size(jwks_pubkey)) {
+          printf("Private keys:\n");
+        }
+        printf("%.*s\n", (int)str_jwks_len, str_jwks);
       }
-      printf("%s\n", str_jwks);
     }
     o_free(str_jwks);
+    str_jwks = NULL;
     json_decref(j_jwks);
+    j_jwks = NULL;
+    str_jwks_len = 0;
   }
   if (r_jwks_size(jwks_pubkey)) {
-    j_jwks = r_jwks_export_to_json_t(jwks_pubkey);
-    str_jwks = json_dumps(j_jwks, JSON_INDENT(indent));
-    if (out_file_public != NULL) {
-      if (write_file_content(out_file_public, str_jwks)) {
-        fprintf(stderr, "Error writing to file %s\n", out_file_public);
-      }
+    if (format == RNBYC_FORMAT_JWK) {
+      j_jwks = r_jwks_export_to_json_t(jwks_pubkey);
+      str_jwks = json_dumps(j_jwks, JSON_INDENT(indent)|JSON_SORT_KEYS);
+      str_jwks_len = o_strlen(str_jwks);
     } else {
-      printf("\nPublic keys:\n%s\n", str_jwks);
+      for (index=0; index<r_jwks_size(jwks_pubkey); index++) {
+        cur_jwk = r_jwks_get_at(jwks_pubkey, index);
+        if (r_jwk_export_to_pem_der(cur_jwk, format==RNBYC_FORMAT_PEM?R_FORMAT_PEM:R_FORMAT_DER, NULL, &out_len, x5u_flags) == RHN_ERROR_PARAM) {
+          if ((out = o_malloc(out_len+1)) != NULL) {
+            if (r_jwk_export_to_pem_der(cur_jwk, format==RNBYC_FORMAT_PEM?R_FORMAT_PEM:R_FORMAT_DER, out, &out_len, x5u_flags) == RHN_OK) {
+              str_jwks = o_realloc(str_jwks, str_jwks_len+out_len);
+              memcpy(str_jwks+str_jwks_len, out, out_len);
+              str_jwks_len += out_len;
+            } else {
+              fprintf(stderr, "Error exporting jwks public\n");
+            }
+          } else {
+            fprintf(stderr, "Error allocating resources\n");
+          }
+          o_free(out);
+          out_len = 0;
+        } else {
+          fprintf(stderr, "Error exporting jwks public\n");
+        }
+        r_jwk_free(cur_jwk);
+      }
+    }
+    if (str_jwks != NULL) {
+      if (out_file_public != NULL) {
+        if (write_file_content(out_file_public, str_jwks, str_jwks_len)) {
+          fprintf(stderr, "Error writing to file %s\n", out_file_public);
+        }
+      } else {
+        printf("\nPublic keys:\n%.*s\n", (int)str_jwks_len, str_jwks);
+      }
     }
     o_free(str_jwks);
     json_decref(j_jwks);
@@ -453,11 +576,12 @@ static void get_jwks_out(json_t * j_arguments, int split_keys, int x5u_flags, in
   r_jwks_free(jwks_pubkey);
 }
 
-static int parse_token(const char * token, int indent, int x5u_flags, const char * str_jwks_pubkey, const char * str_jwks_privkey, int show_header, int show_claims) {
+static int parse_token(const char * token, int indent, int x5u_flags, const char * str_jwks_pubkey, const char * str_jwks_privkey, const char * password, int show_header, int show_claims) {
   int ret = 0, type, res;
   char * content, * str_value;
   jwt_t * jwt = NULL;
   jwks_t * jwks_pubkey = NULL, * jwks_privkey = NULL;
+  jwk_t * jwk_password;
   json_t * j_value;
 
   if (r_jwt_init(&jwt) == RHN_OK) {
@@ -466,30 +590,50 @@ static int parse_token(const char * token, int indent, int x5u_flags, const char
       if (type == R_JWT_TYPE_SIGN || type == R_JWT_TYPE_NESTED_ENCRYPT_THEN_SIGN || type == R_JWT_TYPE_NESTED_SIGN_THEN_ENCRYPT) {
         if (r_jwks_init(&jwks_pubkey) == RHN_OK) {
           if (o_strlen(str_jwks_pubkey) && str_jwks_pubkey[0] == '{') {
-            if (r_jwks_import_from_str(jwks_pubkey, str_jwks_pubkey) != RHN_OK) {
+            if (r_jwks_import_from_json_str(jwks_pubkey, str_jwks_pubkey) != RHN_OK) {
               fprintf(stderr, "Invalid jwks_pubkey\n");
             }
           } else if (o_strlen(str_jwks_pubkey)) {
             content = get_file_content(str_jwks_pubkey);
-            if (r_jwks_import_from_str(jwks_pubkey, content) != RHN_OK) {
-              fprintf(stderr, "Invalid jwks_pubkey path\n");
+            if (r_jwks_import_from_json_str(jwks_pubkey, content) != RHN_OK) {
+              fprintf(stderr, "Invalid jwks_pubkey path or content\n");
             }
             o_free(content);
+          } else if (o_strlen(password)) {
+            r_jwk_init(&jwk_password);
+            if (r_jwk_import_from_password(jwk_password, password) != RHN_OK) {
+              fprintf(stderr, "Error parsing password\n");
+            } else {
+              if (r_jwks_append_jwk(jwks_pubkey, jwk_password) != RHN_OK) {
+                fprintf(stderr, "Error importing password\n");
+              }
+            }
+            r_jwk_free(jwk_password);
           }
         }
       }
       if (type == R_JWT_TYPE_ENCRYPT || type == R_JWT_TYPE_NESTED_ENCRYPT_THEN_SIGN || type == R_JWT_TYPE_NESTED_SIGN_THEN_ENCRYPT) {
         if (r_jwks_init(&jwks_privkey) == RHN_OK) {
           if (o_strlen(str_jwks_privkey) && str_jwks_privkey[0] == '{') {
-            if (r_jwks_import_from_str(jwks_privkey, str_jwks_privkey) != RHN_OK) {
+            if (r_jwks_import_from_json_str(jwks_privkey, str_jwks_privkey) != RHN_OK) {
               fprintf(stderr, "Invalid jwks_privkey\n");
             }
           } else if (o_strlen(str_jwks_privkey)) {
             content = get_file_content(str_jwks_privkey);
-            if (r_jwks_import_from_str(jwks_privkey, content) != RHN_OK) {
-              fprintf(stderr, "Invalid jwks_privkey path\n");
+            if (r_jwks_import_from_json_str(jwks_privkey, content) != RHN_OK) {
+              fprintf(stderr, "Invalid jwks_privkey path or content\n");
             }
             o_free(content);
+          } else if (o_strlen(password)) {
+            r_jwk_init(&jwk_password);
+            if (r_jwk_import_from_password(jwk_password, password) != RHN_OK) {
+              fprintf(stderr, "Error parsing password\n");
+            } else {
+              if (r_jwks_append_jwk(jwks_privkey, jwk_password) != RHN_OK) {
+                fprintf(stderr, "Error importing password\n");
+              }
+            }
+            r_jwk_free(jwk_password);
           }
         }
       }
@@ -530,7 +674,7 @@ static int parse_token(const char * token, int indent, int x5u_flags, const char
       }
       if (show_header) {
         j_value = r_jwt_get_full_header_json_t(jwt);
-        str_value = json_dumps(j_value, JSON_INDENT(indent));
+        str_value = json_dumps(j_value, JSON_INDENT(indent)|JSON_SORT_KEYS);
         printf("%s\n", str_value);
         o_free(str_value);
         json_decref(j_value);
@@ -539,7 +683,7 @@ static int parse_token(const char * token, int indent, int x5u_flags, const char
         str_value = NULL;
         j_value = r_jwt_get_full_claims_json_t(jwt);
         if (j_value != NULL) {
-          str_value = json_dumps(j_value, JSON_INDENT(indent));
+          str_value = json_dumps(j_value, JSON_INDENT(indent)|JSON_SORT_KEYS);
           printf("%s\n", str_value);
           o_free(str_value);
           json_decref(j_value);
@@ -557,42 +701,63 @@ static int parse_token(const char * token, int indent, int x5u_flags, const char
   return ret;
 }
 
-static int serialize_token(const char * claims, int x5u_flags, const char * str_jwks_pubkey, const char * str_jwks_privkey, const char * alg, const char * enc, const char * enc_alg) {
+static int serialize_token(const char * claims, int x5u_flags, const char * str_jwks_pubkey, const char * str_jwks_privkey, const char * password, const char * alg, const char * enc, const char * enc_alg) {
   jwt_t * jwt = NULL;
   jwks_t * jwks_pubkey = NULL, * jwks_privkey = NULL;
+  jwk_t * jwk_password;
   char * token = NULL, * content = NULL;
   int ret = 0;
-  
+
   if (r_jwt_init(&jwt) == RHN_OK) {
     if (r_jwt_set_full_claims_json_str(jwt, claims) == RHN_OK) {
       if (r_jwks_init(&jwks_pubkey) == RHN_OK) {
         if (o_strlen(str_jwks_pubkey) && str_jwks_pubkey[0] == '{') {
-          if (r_jwks_import_from_str(jwks_pubkey, str_jwks_pubkey) != RHN_OK) {
+          if (r_jwks_import_from_json_str(jwks_pubkey, str_jwks_pubkey) != RHN_OK) {
             fprintf(stderr, "Invalid jwks_pubkey\n");
             ret = EINVAL;
           }
         } else if (o_strlen(str_jwks_pubkey)) {
           content = get_file_content(str_jwks_pubkey);
-          if (r_jwks_import_from_str(jwks_pubkey, content) != RHN_OK) {
-            fprintf(stderr, "Invalid jwks_pubkey path\n");
+          if (r_jwks_import_from_json_str(jwks_pubkey, content) != RHN_OK) {
+            fprintf(stderr, "Invalid jwks_pubkey path or content\n");
             ret = EAGAIN;
           }
           o_free(content);
+        } else if (o_strlen(password) && o_strlen(enc_alg)) {
+          r_jwk_init(&jwk_password);
+          if (r_jwk_import_from_password(jwk_password, password) != RHN_OK) {
+            fprintf(stderr, "Error parsing password\n");
+          } else {
+            if (r_jwks_append_jwk(jwks_pubkey, jwk_password) != RHN_OK) {
+              fprintf(stderr, "Error importing password\n");
+            }
+          }
+          r_jwk_free(jwk_password);
         }
       }
       if (r_jwks_init(&jwks_privkey) == RHN_OK) {
         if (o_strlen(str_jwks_privkey) && str_jwks_privkey[0] == '{') {
-          if (r_jwks_import_from_str(jwks_privkey, str_jwks_privkey) != RHN_OK) {
+          if (r_jwks_import_from_json_str(jwks_privkey, str_jwks_privkey) != RHN_OK) {
             fprintf(stderr, "Invalid jwks_privkey\n");
             ret = EINVAL;
           }
         } else if (o_strlen(str_jwks_privkey)) {
           content = get_file_content(str_jwks_privkey);
-          if (r_jwks_import_from_str(jwks_privkey, content) != RHN_OK) {
-            fprintf(stderr, "Invalid jwks_privkey path\n");
+          if (r_jwks_import_from_json_str(jwks_privkey, content) != RHN_OK) {
+            fprintf(stderr, "Invalid jwks_privkey path or content\n");
             ret = EAGAIN;
           }
           o_free(content);
+        } else if (o_strlen(password) && o_strlen(alg)) {
+          r_jwk_init(&jwk_password);
+          if (r_jwk_import_from_password(jwk_password, password) != RHN_OK) {
+            fprintf(stderr, "Error parsing password\n");
+          } else {
+            if (r_jwks_append_jwk(jwks_privkey, jwk_password) != RHN_OK) {
+              fprintf(stderr, "Error importing password\n");
+            }
+          }
+          r_jwk_free(jwk_password);
         }
       }
       if (jwks_pubkey != NULL) {
@@ -634,6 +799,8 @@ static int serialize_token(const char * claims, int x5u_flags, const char * str_
           token = r_jwt_serialize_encrypted(jwt, NULL, x5u_flags);
         } else if (!r_jwks_size(jwks_pubkey) && r_jwks_size(jwks_privkey)) {
           token = r_jwt_serialize_signed(jwt, NULL, x5u_flags);
+        } else if (o_strlen(password)) {
+          token = r_jwt_serialize_encrypted(jwt, NULL, x5u_flags);
         } else {
           r_jwt_set_sign_alg(jwt, R_JWA_ALG_NONE);
           token = r_jwt_serialize_signed(jwt, NULL, x5u_flags);
@@ -641,12 +808,14 @@ static int serialize_token(const char * claims, int x5u_flags, const char * str_
       }
       if (token == NULL) {
         fprintf(stderr, "Error serializing token\n");
+        ret = EINVAL;
       } else {
         printf("%s\n", token);
       }
       o_free(token);
     } else {
       fprintf(stderr, "Error setting JSON claims\n");
+      ret = EINVAL;
     }
   }
   r_jwt_free(jwt);
@@ -656,23 +825,25 @@ static int serialize_token(const char * claims, int x5u_flags, const char * str_
 }
 
 int main (int argc, char ** argv) {
-  int next_option, 
-      action = R_ACTION_NONE, 
-      ret = 0, 
-      has_stdin = 0, 
-      split_keys = 0, 
+  int next_option,
+      action = R_ACTION_NONE,
+      ret = 0,
+      has_stdin = 0,
+      split_keys = 0,
       show_header = 0,
       show_claims = 1,
       x5u_flags = 0,
-      debug_mode = 0;
-  const char * short_options = "j::g:i::f:k:a:e:l:o:p:n:x::t:s:H:C:K:P:u:v::h::d::";
-  char * out_file = NULL, 
-       * out_file_public = NULL, 
-       * parsed_token = NULL, 
-       * str_token_public_key = NULL, 
-       * str_token_private_key = NULL, 
-       * alg = NULL, 
-       * enc = NULL, 
+      debug_mode = 0,
+      format = RNBYC_FORMAT_JWK;
+  const char * short_options = "j::g:i::f:k:a:e:l:o:p:n:F:x::t:s:H::C:K:P:W:u:v::h::d::";
+  char * out_file = NULL,
+       * out_file_public = NULL,
+       * parsed_token = NULL,
+       * str_token_public_key = NULL,
+       * str_token_private_key = NULL,
+       * password = NULL,
+       * alg = NULL,
+       * enc = NULL,
        * enc_alg = NULL,
        * claims = NULL;
   static const struct option long_options[]= {
@@ -687,13 +858,15 @@ int main (int argc, char ** argv) {
     {"out-file", required_argument, NULL, 'o'},
     {"out-file-public", required_argument, NULL, 'p'},
     {"indent", required_argument, NULL, 'n'},
+    {"format", no_argument, NULL, 'F'},
     {"split", no_argument, NULL, 'x'},
     {"parse-token", required_argument, NULL, 't'},
     {"serialize-token", required_argument, NULL, 's'},
-    {"header", required_argument, NULL, 'H'},
+    {"header", no_argument, NULL, 'H'},
     {"claims", required_argument, NULL, 'C'},
     {"public-key", required_argument, NULL, 'P'},
     {"private-key", required_argument, NULL, 'K'},
+    {"password", required_argument, NULL, 'W'},
     {"x5u-flags", required_argument, NULL, 'u'},
     {"version", no_argument, NULL, 'v'},
     {"help", no_argument, NULL, 'h'},
@@ -706,7 +879,7 @@ int main (int argc, char ** argv) {
 
   do {
     next_option = getopt_long(argc, argv, short_options, long_options, NULL);
-    
+
     switch (next_option) {
       case 'j':
         if (action == R_ACTION_NONE) {
@@ -724,7 +897,12 @@ int main (int argc, char ** argv) {
               fprintf(stderr, "--generate: Invalid argument\n");
               ret = EINVAL;
             }
-          } else if (0 == o_strcasecmp("ECDSA256", optarg) || 0 == o_strcasecmp("ECDSA384", optarg) || 0 == o_strcasecmp("ECDSA512", optarg) || 0 == o_strcasecmp("EDDSA", optarg)) {
+          } else if (0 == o_strcasecmp("EC256", optarg) ||
+                     0 == o_strcasecmp("EC384", optarg) ||
+                     0 == o_strcasecmp("EC521", optarg) ||
+                     0 == o_strcasecmp("Ed25519", optarg) ||
+                     0 == o_strcasecmp("X25519", optarg) ||
+                     0 == o_strcasecmp("X448", optarg)) {
             json_array_append_new(j_arguments, json_pack("{ssss}", "source", "generate", "type", optarg));
           } else if (0 == o_strncasecmp(optarg, "oct", o_strlen("oct"))) {
             if (o_strlen(optarg) == o_strlen("oct")) {
@@ -776,6 +954,23 @@ int main (int argc, char ** argv) {
         out_file_public = o_strdup(optarg);
         split_keys = 1;
         break;
+      case 'F':
+        if (action == R_ACTION_JWKS_OUT) {
+          if (0 == o_strncasecmp(optarg, "JWK", o_strlen("JWK"))) {
+            format = RNBYC_FORMAT_JWK;
+          } else if (0 == o_strncasecmp(optarg, "PEM", o_strlen("PEM"))) {
+            format = RNBYC_FORMAT_PEM;
+          } else if (0 == o_strncasecmp(optarg, "DER", o_strlen("DER"))) {
+            format = RNBYC_FORMAT_DER;
+          } else {
+            fprintf(stderr, "--format: Invalid format\n");
+            ret = EINVAL;
+          }
+        } else {
+          fprintf(stderr, "--format: argument incompatible with parse or serialize action\n");
+          ret = EINVAL;
+        }
+        break;
       case 'x':
         split_keys = 1;
         break;
@@ -807,11 +1002,7 @@ int main (int argc, char ** argv) {
         claims = o_strdup(optarg);
         break;
       case 'H':
-        if (0 == o_strcasecmp("false", optarg) || 0 == o_strcasecmp("no", optarg) || 0 == o_strcmp("0", optarg)) {
-          show_header = 0;
-        } else {
-          show_header = 1;
-        }
+        show_header = 1;
         break;
       case 'C':
         if (0 == o_strcasecmp("false", optarg) || 0 == o_strcasecmp("no", optarg) || 0 == o_strcmp("0", optarg)) {
@@ -826,6 +1017,9 @@ int main (int argc, char ** argv) {
       case 'P':
         str_token_public_key = o_strdup(optarg);
         break;
+      case 'W':
+        password = o_strdup(optarg);
+        break;
       case 'u':
         if (o_strcasestr(optarg, "cert") != NULL) {
           x5u_flags |= R_FLAG_IGNORE_SERVER_CERTIFICATE;
@@ -852,25 +1046,25 @@ int main (int argc, char ** argv) {
         break;
     }
   } while (next_option != -1 && !ret);
-  
+
   if (debug_mode) {
     y_init_logs("rnbyc", Y_LOG_MODE_CONSOLE, Y_LOG_LEVEL_DEBUG, NULL, "Starting rnbyc debug mode");
   }
-  
+
   if (!ret) {
     if (action == R_ACTION_JWKS_OUT) {
-      get_jwks_out(j_arguments, split_keys, x5u_flags, indent, out_file, out_file_public);
+      get_jwks_out(j_arguments, split_keys, x5u_flags, indent, format, out_file, out_file_public);
     } else if (action == R_ACTION_PARSE_TOKEN) {
-      ret = parse_token(parsed_token, indent, x5u_flags, str_token_public_key, str_token_private_key, show_header, show_claims);
+      ret = parse_token(parsed_token, indent, x5u_flags, str_token_public_key, str_token_private_key, password, show_header, show_claims);
     } else if (action == R_ACTION_SERIALIZE_TOKEN) {
-      ret = serialize_token(claims, x5u_flags, str_token_public_key, str_token_private_key, alg, enc, enc_alg);
+      ret = serialize_token(claims, x5u_flags, str_token_public_key, str_token_private_key, password, alg, enc, enc_alg);
     } else {
       ret = EINVAL;
       fprintf(stderr, "Please epecify an action\n");
       print_help(stderr);
     }
   }
-  
+
   json_decref(j_arguments);
   o_free(out_file);
   o_free(out_file_public);
@@ -881,7 +1075,8 @@ int main (int argc, char ** argv) {
   o_free(claims);
   o_free(str_token_private_key);
   o_free(str_token_public_key);
-  
+  o_free(password);
+
   if (debug_mode) {
     y_close_logs();
   }
